# Cloud Connector

The `cloud-connector` is a set of services that build an "authenticated channel" between
an NSX (on-premises) deployment and the Lastline cloud.

## Configuration

The cloud to which the `cloud-connector` is connected to is configured via
- `cloudConfiguration.cloudRegion.region`: a region name, such as `west.us` that uniquely
  identifies a Lastline cloud region,
- `cloudConfiguration.cloudRegion.regionName`: a human-readable representation of the
  `region` parameter, and
- `cloudConfiguration.cloudRegion.fqdn`: the FQDN that must be used to reach cloud APIs
  for the selected `region`.

All values are set via `values.yaml` or deployment settings (`--set...`) during installation.
On upgrade, the chart will attempt to do a lookup of the values from a *configmap* and fail if
no values are provided.

Possible values can be retrieved from any Lastline Cloud using

```bash
curl -X GET 'https://nsx.lastline.com/nsx/cloud-connector/api/v1/papi/accounting/nsx/get_cloud_regions.json' | python -m json.tool
```

An example response for above API call offers two options for the cloud
region (`west.us` and `nl.emea`):

```json
{
    "data": [
        {
            "fqdn": "nsx.west.us.lastline.com",
            "region": "west.us",
            "region_name": "United States 1"
        },
        {
            "fqdn": "nsx.nl.emea.lastline.com",
            "region": "nl.emea",
            "region_name": "European Union 1"
        }
    ],
    "success": 1
}
```

A configuration override yaml for the above `west.us` region would look like this:

```yaml
cloudConfiguration:
  cloudRegion:
    region: "west.us"
    regionName: "United States 1"
    fqdn: "nsx.west.us.lastline.com"
```

A configuration CLI for the above `west.us` region would look like this:

```bash
helm install \
    cloud-connector \
    ./cloud-connector/ \
    --set 'cloudConfiguration.cloudRegion.region=west.us' \
    --set 'cloudConfiguration.cloudRegion.regionName=United States 1' \
    --set 'cloudConfiguration.cloudRegion.fqdn=nsx.west.us.lastline.com'
```

**NOTE:** Since The configuration is optional in the *upgrade* phase, we cannot make use
of a JSON schema validation for the above values. The `cloud-connector-precheck` helm chart,
on the other hand, is used only during the install phase and contains a schema validation
file for reference.

## Deployment

The `cloud-connector` requires a Lastline cloud license, which it registers automatically on
deployment (via a helm *pre-install* hook). A requirement is that the base Kubernetes platform is
connected to an NSX Manager with a license for an advanced SKU (e.g., *NSX Data Center Advanced +
NSX Firewall ATP Add on* or *NSX Datacenter Evaluation*).

During registration, the NSX license is retrieved from the NSX Manager (via an API request) and
pushed to the Lastline cloud, which evaluates if the license applies for a sign-up, and - if
so - responds with the details of the created Lastline license.

The registration job will write the cloud license credentials as Kubernetes `secret`. At
the same time, the helm chart will write the cloud settings (cloud `region`, `region_name`, and
`fqdn`) to a Kubernetes `configmap`. The `secret` and `configmap` can then be used by other
verticals (if necessary).

## Lifecycle

The `cloud-connector` is (conceptually) not a feature in itself. Instead, it is a dependency
for other features that need to communicate with the Lastline cloud. It has to be deployed as
a standalone chart (as opposed to being used as helm *dependency*), however, because we want to
be able to share a deployment across multiple features and to avoid binding the lifecycle
directly to a specific feature.

When the `cloud-connector` is uninstalled, a helm *pre-uninstall* hook will notify the Lastline
cloud that the license is no-longer needed. The cloud, in turn, will invalidate (expire) the
license and mark the cloud data associated with the account for deletion.

To determine if it is safe to uninstall the `cloud-connector`, we can make use of the *consumer
CRD*: each vertical that depends on Cloud Connector should register a CRD instance with the
Kubernetes API. As long as at least one consumer exists, the deployment should remain active.

**NOTE:** It is possible to register the *Cloud Connector* deployment as *consumer* itself. This
ensures that the *Cloud Connector* is not removed, even if no other vertical depends on it.

**NOTE:** Triggering the removal could be done via a Kubernetes operator by monitoring for
removal of *consumer* CRDs. This is currently not done, as we want the `cluster-api` to be in
charge of helm invocations. But, the entire design is already ready for such a move for when
we will be bold enough to go there.

## Components

The `cloud-connector` consists of three components:
- `cloud-connector`: *main* helm chart (this component),
- `cloud-connector-crds`: a helm chart deploying a Custom Resource Definition (CRD) to be used
  for ref-counting the number of verticals that make use of the *Cloud Connector*. If the
  ref-count drops to 0, the main component may be removed. This helm chart is typically deployed
  as helm *dependency* of the *base platform* to ensure the CRD is known for other verticals to
  deploy, and
- `cloud-connector-common`: a helm chart holding shared constants and deploying a CRD for
  ref-counting purposes (see above). This chart is typically used as helm *dependency* of the
  verticals using the *Cloud Connector*.
